#include <stdio.h>
#include <stdlib.h>
#include <netinet/ip.h>     /* struct iphdr */
#include <unistd.h>         /* read */
#include <sys/socket.h>
#include <string.h>         /* strerror */

#include "ospf.h"
#include "chksum.h"
#include "msgtype.h"
#include "authtype.h"
#include "print.h"
#include "init.h"
#include "getaddr.h"
#include "getiface.h"

/* http://www.tenouk.com/Module43a.html */
#define PCKT_LEN 8192

int main (int argc, char ** argv)
{
  struct iphdr * ip;
  struct OSPFheader * header;
  struct OSPFhello * hello;
  char * iface;
  
  uint8_t buffer[PCKT_LEN];
#ifndef _NO_SEND
  int sd;
  struct sockaddr_in sin;
  const int on = 1;
#endif

  uint32_t address, netmask;

  memset(buffer, 0, PCKT_LEN);

  ip = (struct iphdr *) buffer;
  header = (struct OSPFheader *) (buffer + sizeof(struct iphdr));
  hello = (struct OSPFhello *) (buffer + sizeof(struct iphdr) +
			        sizeof(struct OSPFheader));

  if (argc > 1)
    iface = argv[1];
  else
    iface = getDefaultInterface();

  address = getAddress(iface);
  netmask = getNetmask(iface);

  initIP(ip, address);
  initOSPFHeader(header, address);
  initOSPFHello(hello, address, netmask);
  
  ip->tot_len = sizeof(struct iphdr) + sizeof(struct OSPFheader) +
    sizeof(struct OSPFhello); /* single-byte field, so no htons */

  header->length = htons(sizeof(struct OSPFheader) + sizeof(struct OSPFhello));

  ip->check = 0;
  ip->check = chksum(ip, ip->ihl * 4); // interesting fact: do not change byte ordering

  header->chksum = 0;
  header->chksum = chksum(header, ntohs(header->length));

#ifndef _NO_PRINT
  printIP(ip);
  printOSPFHeader(header);
  printOSPFHello(hello);
#endif
  
#ifndef _NO_SEND
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = ip->daddr;

  sd = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
  if (sd < 0)
    {
      perror("socket() error");
      exit(EXIT_FAILURE);
    }

  if (setsockopt(sd, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on)) < 0)
    {
      perror("setsockopt() error");
      exit(EXIT_FAILURE);
    }

  if (sendto(sd, buffer, ip->tot_len, 0, (struct sockaddr *)&sin,
	     sizeof(struct sockaddr)) < 0)
    {
      perror("sendto() failed");
      exit(EXIT_FAILURE);
    }
  
  close(sd);
#endif
  return(EXIT_SUCCESS);
}
