#include <stdio.h>
#include <stdlib.h>         /* calloc */
#include <netinet/ip.h>     /* struct iphdr */
#include <time.h>           /* needed to generate random numbers */
#include <string.h>         /* strerror */
#include <arpa/inet.h>      /* inet_pton */
#include <netdb.h>          /* /etc/protocols */

#include "init.h"
#include "getaddr.h"
#include "options.h"

uint16_t randomIdentifier ()
{
  srand(time(NULL));
  return (uint16_t) rand();
}

void initIP (struct iphdr * ip, uint32_t address)
{
  int status;

  ip->version = 4;                 /* IPv4 */
  ip->ihl     = 5;                 /* No options, header length = 20 bytes */
  ip->tos     = IPTOS_CLASS_CS6;   /* from netinet/ip.h */
  ip->tot_len = UINT16_MAX;        /* initialize to max value */
  ip->id = randomIdentifier();     /* Linux will automatically set this field */
  ip->frag_off = 0;                /* fragmentation is so not cool anymore */
  
  /* From http://tools.ietf.org/html/rfc2328#appendix-A.1
     "To ensure that these packets will not travel multiple hops,
     their IP TTL must be set to 1." */
  ip->ttl = 1;

  /* http://www.iana.org/assignments/protocol-numbers */
  ip->protocol = getprotobyname("ospf")->p_proto; /* I know it's 89... */

  ip->check = 0;                   /* Must be zero during checksum calculation */
  ip->saddr = address;

  /* Destination IP Address */
  if ((status = inet_pton (AF_INET, "224.0.0.5", &(ip->daddr))) != 1)
    {
      fprintf(stderr, "inet_pton error: %s\n", strerror(status));
      exit(EXIT_FAILURE);
    }
}

void initOSPFHeader(struct OSPFheader * header, uint32_t address)
{
  header->version = 2;
  header->msgtype = 1;
  /* Skip length */
  header->rid = address;
  header->areaid  = 0;
  /* Skip checksum */
  header->autype  = 0;
  header->authen  = 0;
}

void initOSPFHello(struct OSPFhello * hello, uint32_t address, uint32_t netmask)
{
  hello->netmask = netmask;
  hello->hellointerval = htons(10);
  hello->options = OSPF_OPTION_E | 0x00;
  hello->priority = 0;
  hello->deadint  = htonl(40);
  hello->dr = address;
  hello->bdr = 0;
}
